@@ -78,6 +78,9 @@ FIELD_TYPES: tuple[FieldType, ...] = (
     FieldType(
         name="GEOGRAPHY", dbapi_type=[DBAPI_TYPE_STRING], pa_type=lambda: pa.string()
     ),
+    FieldType(
+        name="GEOMETRY", dbapi_type=[DBAPI_TYPE_STRING], pa_type=lambda: pa.string()
+    ),
 )
 
 FIELD_NAME_TO_ID: DefaultDict[Any, int] = defaultdict(int)
@@ -220,6 +223,7 @@ PARAMETER_CLIENT_REQUEST_MFA_TOKEN = "CLIENT_REQUEST_MFA_TOKEN"
 PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL = (
     "CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTAIL"
 )
+PARAMETER_QUERY_CONTEXT_CACHE_SIZE = "QUERY_CONTEXT_CACHE_SIZE"
 PARAMETER_TIMEZONE = "TIMEZONE"
 PARAMETER_SERVICE_NAME = "SERVICE_NAME"
 PARAMETER_CLIENT_VALIDATE_DEFAULT_PARAMETERS = "CLIENT_VALIDATE_DEFAULT_PARAMETERS"
