@@ -21,7 +21,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Generator,
     Iterator,
     NamedTuple,
     NoReturn,
@@ -124,10 +123,10 @@ ASYNC_RETRY_PATTERN = [1, 1, 2, 3, 4, 8, 10]
 class ResultMetadata(NamedTuple):
     name: str
     type_code: int
-    display_size: int
-    internal_size: int
-    precision: int
-    scale: int
+    display_size: int | None
+    internal_size: int | None
+    precision: int | None
+    scale: int | None
     is_nullable: bool
 
     @classmethod
@@ -183,12 +182,12 @@ class SnowflakeCursor:
         connection: The connection object by which the cursor was created.
         errorhandle: The class that handles error handling.
         is_file_transfer: Whether, or not the current command is a put, or get.
-
-    TODO:
-        Most of these attributes have no reason to be properties, we could just store them in public variables.
-        Calling a function is expensive in Python and most of these getters are unnecessary.
     """
 
+    # TODO:
+    #    Most of these attributes have no reason to be properties, we could just store them in public variables.
+    #    Calling a function is expensive in Python and most of these getters are unnecessary.
+
     INSERT_SQL_RE = re.compile(r"^insert\s+into", flags=re.IGNORECASE)
     COMMENT_SQL_RE = re.compile(r"/\*.*\*/")
     INSERT_SQL_VALUES_RE = re.compile(
@@ -265,6 +264,7 @@ class SnowflakeCursor:
         self._inner_cursor: SnowflakeCursor | None = None
         self._prefetch_hook = None
         self._rownumber: int | None = None
+        self._cached_last_result: dict | None = None
 
         self.reset()
 
@@ -543,25 +543,27 @@ class SnowflakeCursor:
                 self._timebomb.cancel()
                 logger.debug("cancelled timebomb in finally")
 
+            self._cached_last_result = ret
+
         if "data" in ret and "parameters" in ret["data"]:
-            parameters = ret["data"]["parameters"]
+            parameters = ret["data"].get("parameters", list())
             # Set session parameters for cursor object
             for kv in parameters:
                 if "TIMESTAMP_OUTPUT_FORMAT" in kv["name"]:
                     self._timestamp_output_format = kv["value"]
-                if "TIMESTAMP_NTZ_OUTPUT_FORMAT" in kv["name"]:
+                elif "TIMESTAMP_NTZ_OUTPUT_FORMAT" in kv["name"]:
                     self._timestamp_ntz_output_format = kv["value"]
-                if "TIMESTAMP_LTZ_OUTPUT_FORMAT" in kv["name"]:
+                elif "TIMESTAMP_LTZ_OUTPUT_FORMAT" in kv["name"]:
                     self._timestamp_ltz_output_format = kv["value"]
-                if "TIMESTAMP_TZ_OUTPUT_FORMAT" in kv["name"]:
+                elif "TIMESTAMP_TZ_OUTPUT_FORMAT" in kv["name"]:
                     self._timestamp_tz_output_format = kv["value"]
-                if "DATE_OUTPUT_FORMAT" in kv["name"]:
+                elif "DATE_OUTPUT_FORMAT" in kv["name"]:
                     self._date_output_format = kv["value"]
-                if "TIME_OUTPUT_FORMAT" in kv["name"]:
+                elif "TIME_OUTPUT_FORMAT" in kv["name"]:
                     self._time_output_format = kv["value"]
-                if "TIMEZONE" in kv["name"]:
+                elif "TIMEZONE" in kv["name"]:
                     self._timezone = kv["value"]
-                if "BINARY_OUTPUT_FORMAT" in kv["name"]:
+                elif "BINARY_OUTPUT_FORMAT" in kv["name"]:
                     self._binary_output_format = kv["value"]
             # Set session parameters for connection object
             self._connection._update_parameters(
@@ -710,7 +712,8 @@ class SnowflakeCursor:
             _get_azure_callback: Function to which an Azure GET command should call back to.
             _get_callback_output_stream: The output stream a GET command's callback should report on.
             _show_progress_bar: Whether or not to show progress bar.
-            _statement_params: Extra information that should be sent to Snowflake with query.
+            _statement_params: Extra information that should be sent to Snowflake with query. This dict will not be
+                modified by the connector.
             _is_internal: This flag indicates whether the query is issued internally by the connector.
             _describe_only: If true, the query will not be executed but return the schema/description of this query.
             _no_results: This flag tells the back-end to not return the result, just fire the query and return the
@@ -748,11 +751,14 @@ class SnowflakeCursor:
             logger.warning("execute: no query is given to execute")
             return None
 
-        if _statement_params is None:
-            _statement_params = dict()
-
-        if num_statements:
-            _statement_params["MULTI_STATEMENT_COUNT"] = num_statements
+        _statement_params = _statement_params or dict()
+        # If we need to add another parameter, please consider introducing a dict for all extra params
+        # See discussion in https://github.com/snowflakedb/snowflake-connector-python/pull/1524#discussion_r1174061775
+        if num_statements is not None:
+            _statement_params = {
+                **_statement_params,
+                "MULTI_STATEMENT_COUNT": num_statements,
+            }
 
         kwargs: dict[str, Any] = {
             "timeout": timeout,
@@ -1070,6 +1076,7 @@ class SnowflakeCursor:
             Error.errorhandler_wrapper(
                 self.connection, self, ProgrammingError, errvalue
             )
+        self._cached_last_result = ret
         return self
 
     def fetch_arrow_batches(self) -> Iterator[Table]:
@@ -1119,6 +1126,7 @@ class SnowflakeCursor:
     def abort_query(self, qid: str) -> bool:
         url = f"/queries/{qid}/abort-request"
         ret = self._connection.rest.request(url=url, method="post")
+        self._cached_last_result = ret
         return ret.get("success")
 
     def executemany(
@@ -1238,26 +1246,6 @@ class SnowflakeCursor:
 
         return self
 
-    def _result_iterator(
-        self,
-    ) -> Generator[dict, None, None] | Generator[tuple, None, None]:
-        """Yields the elements from _result and raises an exception when appropriate."""
-        try:
-            for _next in self._result:
-                if isinstance(_next, Exception):
-                    Error.errorhandler_wrapper_from_ready_exception(
-                        self._connection,
-                        self,
-                        _next,
-                    )
-                self._rownumber += 1
-                yield _next
-        except TypeError as err:
-            if self._result_state == ResultState.DEFAULT:
-                raise err
-            else:
-                yield None
-
     def fetchone(self) -> dict | tuple | None:
         """Fetches one row."""
         if self._prefetch_hook is not None:
@@ -1265,10 +1253,23 @@ class SnowflakeCursor:
         if self._result is None and self._result_set is not None:
             self._result = iter(self._result_set)
             self._result_state = ResultState.VALID
+
         try:
-            return next(self._result_iterator())
-        except StopIteration:
-            return None
+            _next = next(self._result, None)
+            if isinstance(_next, Exception):
+                Error.errorhandler_wrapper_from_ready_exception(
+                    self._connection,
+                    self,
+                    _next,
+                )
+            if _next is not None:
+                self._rownumber += 1
+            return _next
+        except TypeError as err:
+            if self._result_state == ResultState.DEFAULT:
+                raise err
+            else:
+                return None
 
     def fetchmany(self, size: int | None = None) -> list[tuple] | list[dict]:
         """Fetches the number of specified rows."""
@@ -1312,6 +1313,8 @@ class SnowflakeCursor:
         to any of the fetch*() methods will return rows from the next query's set of results. Returns None if no more
         query results are available.
         """
+        if self._prefetch_hook is not None:
+            self._prefetch_hook()
         self.reset()
         if self._multi_statement_resultIds:
             self.query_result(self._multi_statement_resultIds[0])
@@ -1363,11 +1366,11 @@ class SnowflakeCursor:
 
     def __iter__(self) -> Iterator[dict] | Iterator[tuple]:
         """Iteration over the result set."""
-        # set _result if _result_set is not None
-        if self._result is None and self._result_set is not None:
-            self._result = iter(self._result_set)
-            self._result_state = ResultState.VALID
-        return self._result_iterator()
+        while True:
+            _next = self.fetchone()
+            if _next is None:
+                break
+            yield _next
 
     def __cancel_query(self, query) -> None:
         if self._sequence_counter >= 0 and not self.is_closed():
@@ -1459,6 +1462,7 @@ class SnowflakeCursor:
             ):
                 url = f"/queries/{sfqid}/result"
                 ret = self._connection.rest.request(url=url, method="get")
+                self._cached_last_result = ret
                 if "data" in ret and "resultIds" in ret["data"]:
                     self._init_multi_statement_results(ret["data"])
 
