@@ -32,13 +32,15 @@ T = TypeVar("T", bound=collections.abc.Sequence)
 logger = getLogger(__name__)
 
 
-def chunk_helper(lst: T, n: int) -> Iterator[tuple[int, T]]:
+def chunk_helper(
+    lst: pandas.DataFrame, n: int
+) -> Iterator[tuple[int, pandas.DataFrame]]:
     """Helper generator to chunk a sequence efficiently with current index like if enumerate was called on sequence."""
     if len(lst) == 0:
         yield 0, lst
         return
     for i in range(0, len(lst), n):
-        yield int(i / n), lst[i : i + n]
+        yield int(i / n), lst.iloc[i : i + n]
 
 
 def build_location_helper(
@@ -170,6 +172,19 @@ def write_pandas(
     if chunk_size is None:
         chunk_size = len(df)
 
+    if not (
+        isinstance(df.index, pandas.RangeIndex)
+        and 1 == df.index.step
+        and 0 == df.index.start
+    ):
+        warnings.warn(
+            f"Pandas Dataframe has non-standard index of type {str(type(df.index))} which will not be written."
+            f" Consider changing the index to pd.RangeIndex(start=0,...,step=1) or "
+            f"call reset_index() to keep index as column(s)",
+            UserWarning,
+            stacklevel=2,
+        )
+
     cursor = conn.cursor()
     stage_location = build_location_helper(
         database=database,
@@ -204,12 +219,13 @@ def write_pandas(
     # see (https://docs.snowflake.com/en/user-guide/script-data-load-transform-parquet.html)
     if quote_identifiers:
         quote = '"'
-        columns = '"' + '","'.join(list(df.columns)) + '"'
-        parquet_columns = "$1:" + ",$1:".join(f'"{c}"' for c in df.columns)
+        # if the column name contains a double quote, we need to escape it by replacing with two double quotes
+        # https://docs.snowflake.com/en/sql-reference/identifiers-syntax#double-quoted-identifiers
+        snowflake_column_names = [str(c).replace('"', '""') for c in df.columns]
     else:
         quote = ""
-        columns = ",".join(list(df.columns))
-        parquet_columns = "$1:" + ",$1:".join(df.columns)
+        snowflake_column_names = list(df.columns)
+    columns = quote + f"{quote},{quote}".join(snowflake_column_names) + quote
 
     def drop_object(name: str, object_type: str) -> None:
         drop_sql = f"DROP {object_type.upper()} IF EXISTS {name} /* Python:snowflake.connector.pandas_tools.write_pandas() */"
@@ -240,7 +256,10 @@ def write_pandas(
         # so we have to iterate through the dataframe columns to make sure we create the table with its
         # columns in order
         create_table_columns = ", ".join(
-            [f"{quote}{c}{quote} {column_type_mapping[c]}" for c in df.columns]
+            [
+                f"{quote}{snowflake_col}{quote} {column_type_mapping[col]}"
+                for snowflake_col, col in zip(snowflake_column_names, df.columns)
+            ]
         )
 
         target_table_location = build_location_helper(
@@ -257,6 +276,11 @@ def write_pandas(
         )
         logger.debug(f"auto creating table with '{create_table_sql}'")
         cursor.execute(create_table_sql, _is_internal=True)
+        # need explicit casting when the underlying table schema is inferred
+        parquet_columns = "$1:" + ",$1:".join(
+            f"{quote}{snowflake_col}{quote}::{column_type_mapping[col]}"
+            for snowflake_col, col in zip(snowflake_column_names, df.columns)
+        )
     else:
         target_table_location = build_location_helper(
             database=database,
@@ -264,13 +288,16 @@ def write_pandas(
             name=table_name,
             quote_identifiers=quote_identifiers,
         )
+        parquet_columns = "$1:" + ",$1:".join(
+            f"{quote}{snowflake_col}{quote}" for snowflake_col in snowflake_column_names
+        )
 
     try:
         copy_into_sql = (
             f"COPY INTO {target_table_location} /* Python:snowflake.connector.pandas_tools.write_pandas() */ "
             f"({columns}) "
             f"FROM (SELECT {parquet_columns} FROM @{stage_location}) "
-            f"FILE_FORMAT=(TYPE=PARQUET COMPRESSION={compression_map[compression]}) "
+            f"FILE_FORMAT=(TYPE=PARQUET COMPRESSION={compression_map[compression]}{' BINARY_AS_TEXT=FALSE' if auto_create_table or overwrite else ''}) "
             f"PURGE=TRUE ON_ERROR={on_error}"
         )
         logger.debug(f"copying into with '{copy_into_sql}'")
